import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:flutter_svg/flutter_svg.dart';
import 'package:storybook_flutter/storybook_flutter.dart';

import '../../../matule_uikit.dart';

abstract class BaseButtonWidget extends StatelessWidget {
  final CustomTheme theme;
  final double width;
  final double height;
  final BorderRadius borderRadius;
  final EdgeInsets padding;
  final Function()? onTap;
  final Widget? child;

  const BaseButtonWidget({
    super.key,
    required this.theme,
    required this.width,
    required this.height,
    required this.onTap,
    required this.child,
    required this.borderRadius,
    required this.padding,
  });

  @override
  Widget build(BuildContext context) {
    var theme = CustomTheme.of(context);
    return SizedBox(
      width: width,
      height: height,
      child: FilledButton(
        onPressed: onTap,
        style: FilledButton.styleFrom(
          padding: padding,
          backgroundColor: theme.palette.inputBackground,
          shape: RoundedRectangleBorder(borderRadius: borderRadius),
        ),
        child: child,
      ),
    );
  }
}

abstract class BaseStateButtonWidget extends StatefulWidget {
  final CustomTheme theme;
  final Function()? onTap;
  final Widget? child;
  final Color backgroundColor;
  final Color disabledBackgroundColor;
  final Color strokeColor;
  final String text;
  final Color textColor;

  const BaseStateButtonWidget({
    super.key,
    required this.theme,
    required this.onTap,
    required this.child,
    required this.text,
    required this.textColor,
    required this.backgroundColor,
    required this.disabledBackgroundColor,
    required this.strokeColor,
  });

  double get _width;

  double get _height;
}

class BubbleButtonWidget extends BaseButtonWidget {
  const BubbleButtonWidget({
    super.key,
    required super.theme,
    required super.width,
    required super.height,
    required super.onTap,
    required super.child,
    required super.borderRadius,
    required super.padding,
  });

  BubbleButtonWidget.back({
    super.key,
    required super.theme,
    required super.onTap,
  }) : super(
         width: 32.r,
         height: 32.r,
         padding: EdgeInsets.all(6.r),
         borderRadius: BorderRadius.circular(8.r),
         child: SvgPicture.asset(
           "packages/matule_uikit/assets/icons/chevron-down.svg",
           width: 20.r,
           height: 20.r,
         ),
       );

  BubbleButtonWidget.filter({
    super.key,
    required super.theme,
    required super.onTap,
  }) : super(
         width: 48.r,
         height: 48.r,
         padding: EdgeInsets.all(14.r),
         borderRadius: BorderRadius.circular(10.r),
         child: SvgPicture.asset(
           "packages/matule_uikit/assets/icons/filter.svg",
           width: 20.r,
           height: 20.r,
         ),
       );

  static Story get story => Story(
    name: "BubbleButton",
    builder: (BuildContext context) {
      var theme = CustomTheme.of(context);
      onTap() {
        debugPrint("BubbleButton pressed!");
      }

      int type = context.knobs.options(
        label: "Type",
        initial: 0,
        options: [
          Option(label: "Back", value: 0),
          Option(label: "Filter", value: 1),
        ],
      );
      switch (type) {
        case 0:
          return BubbleButtonWidget.back(theme: theme, onTap: onTap);
        case _:
          return BubbleButtonWidget.filter(theme: theme, onTap: onTap);
      }
    },
  );
}
